package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.IMFeign;
import com.steerinfo.dil.mapper.QmsQueueListMapper;
import com.steerinfo.dil.mapper.QmsQueueResultMapper;
import com.steerinfo.dil.mapper.QmsQueueSpellingListMapper;
import com.steerinfo.dil.mapper.QueuingRulesMapper;
import com.steerinfo.dil.model.QmsQueueList;
import com.steerinfo.dil.model.QmsQueueSpellingList;
import com.steerinfo.dil.service.IQueuingRulesService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 排队规则
 * @ author    :TXF
 * @ time      :2021/10/16 10:04
 */

@Service
public class QueuingRulesServiceImpl implements IQueuingRulesService {

    @Resource
    private QueuingRulesMapper queuingRulesMapper;

    @Autowired
    private QmsQueueListMapper qmsQueueListMapper;

    @Autowired
    private QmsQueueSpellingListMapper qmsQueueSpellingListMapper;

    @Autowired
    private IMFeign imFeign;
    /**
     * 定时检查  钢材科已确认进厂之后  30分钟没有结束 自动回滚到队列末尾，移除可进厂时间
     */
    @Scheduled(fixedRate = 1000 * 60)
    public void checkoutQueueList(){
        long nowTime = System.currentTimeMillis();
        //查询所有单拼排队网格已经确认进厂的记录
        List<Map<String, Object>> listListMap = queuingRulesMapper.getAllSureTimeList();
        for (Map<String, Object> map : listListMap) {
            if((nowTime - ((Date) map.get("entrySureTime")).getTime()) > 1800000){
                //如果超过勾选时间30分钟 自动回滚到队列最后
                BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));
                Integer maxListNodeOrder = qmsQueueListMapper.getMaxListNodeOrder(map);
                queuingRulesMapper.updateListToLast(listId, maxListNodeOrder + 1);
                //移除可进厂资格
                queuingRulesMapper.updateEnFactory(DataChange.dataToBigDecimal(map.get("resultTotalId")));
                if((nowTime - ((Date) map.get("insertTime")).getTime()) > 86400000){
                    //如果大于一天则直接删除队列信息
                    qmsQueueListMapper.deleteByPrimaryKey(listId);
                    queuingRulesMapper.updateQQRRemoveList(DataChange.dataToBigDecimal(map.get("resultId")));
                }
            }
        }
        //查询所有多拼排队网格已经确认进厂的记录
        List<Map<String, Object>> allSureTimeSpellingList = queuingRulesMapper.getAllSureTimeSpellingList();
        for (Map<String, Object> map : allSureTimeSpellingList) {
            if ((nowTime - ((Date) map.get("entrySureTime")).getTime()) > 1800000) {
                BigDecimal spellingResultId = DataChange.dataToBigDecimal(map.get("spellingResultId"));
                //如果超过勾选时间30分钟 自动回滚到队列最后
                queuingRulesMapper.updateSpellingListToLast(spellingResultId);
                //移除可进厂资格
                queuingRulesMapper.updateEnFactory(DataChange.dataToBigDecimal(map.get("resultTotalId")));
                if((nowTime - ((Date) map.get("spellingSureTime")).getTime()) > 86400000){
                    //如果大于一天则直接删除队列信息
                    qmsQueueSpellingListMapper.deleteByPrimaryKey(spellingResultId);
                    queuingRulesMapper.updateQQRRemoveList(DataChange.dataToBigDecimal(map.get("resultId")));
                }
            }
        }
    }

    /**
     * 遍历排队所有未勾选确认进厂的排队数据，如果已经超过一天则踢出队列
     */
    @Scheduled(fixedRate = 1000 * 60 * 30)
    public void checkoutQueue(){
        long nowTime = System.currentTimeMillis();
        List<Map<String, Object>> listMap = queuingRulesMapper.getAllNotSureEnFactoryList();
        for (Map<String, Object> map : listMap) {
            if((nowTime - ((Date) map.get("insertTime")).getTime()) > 86400000){
                //踢出队列
                qmsQueueListMapper.deleteByPrimaryKey(DataChange.dataToBigDecimal(map.get("listId")));
                //修改信息
                queuingRulesMapper.updateQQRRemoveList(DataChange.dataToBigDecimal(map.get("resultId")));
            }
        }
        List<Map<String, Object>> spellingListMap = queuingRulesMapper.getAllNotSureEnFactorySpellingList();
        for (Map<String, Object> map : spellingListMap) {
            if((nowTime - ((Date) map.get("spellingSureTime")).getTime()) > 86400000){
                //如果大于一天则直接删除队列信息
                qmsQueueSpellingListMapper.deleteByPrimaryKey(DataChange.dataToBigDecimal(map.get("spellingResultId")));
                queuingRulesMapper.updateQQRRemoveList(DataChange.dataToBigDecimal(map.get("resultId")));
            }
        }
    }


    /**
     * 推送消息给websocket
     */
    public void pushMesToWebsocket(ArrayList<Object> list) {
        HashMap<Object, Object> sendMap = new HashMap<>();
        sendMap.put("messages", list);
        imFeign.sendToUser(sendMap);
    }

    /*
    //添加消息实体
        HashMap<Object, Object> mesMap = new HashMap<>();
        mesMap.put("messageType", 4); //用于进厂后 刷新消息
        mesMap.put("sendPerson", "system");
        mesMap.put("receivePerson", capacityNumber);
        mesMap.put("messageContent", pushMes + new Date());
        mesMap.put("createTime", new Date());
        ArrayList<Object> list = new ArrayList<>();
        list.add(mesMap);
        //调用websocket接口推送
        HashMap<Object, Object> sendMap = new HashMap<>();
        //将消息实体放入list中存到map中
        sendMap.put("messages", list);
        imFeign.sendToUser(sendMap);
     */
    /*=========================进厂排队规则=========================*/

    /**
     * 通过物资Id查询网格ID
     * @param materialId
     * @return
     */
//    public BigDecimal calculateGridId(BigDecimal materialId){
//        Map<String, Object> map = new HashMap<>();
//        //通过物资Id 和 进厂类型 查询 门岗ID
//        map.put("materialId", materialId);
//        map.put("type", 0); //匹配物资可进厂的门岗
//        List<Map<String, Object>> gatepostMesList = queuingRulesMapper.queryGatepostByMaterialId(map); //可能会有多个门岗可入
//        for (Map<String, Object> mes : gatepostMesList) {
//            //从数据库中获取门岗开始时间结束时间 若当前时间满足该门岗进门时间 则下放 暂不考虑门岗优先级
//            boolean judgeTime = judgeTime((String)mes.get("startTime"), (String)mes.get("endTime"));
//            if(judgeTime){
//                mes.put("gridType", 3);
//                //通过匹配出来的门岗ID获取网格ID
//                return getGridIdByGatepostIdAndGridType(mes);
//            }
//        }
//        return null;
//    }

    /**
     * 判断当前时间是否在时间区间范围内
     * @param startTime
     * @param endTime
     * @return
     */
    public boolean judgeTime(String startTime, String endTime){
        SimpleDateFormat sdf = new SimpleDateFormat("HHmmss");
        String format = sdf.format(new Date());
        int begin = Integer.parseInt(startTime);
        int end = Integer.parseInt(endTime);
        int now = Integer.parseInt(format);
        if(begin < end){
            return now < end && now >= begin;
        }else {
            return now < end || now >= begin;
        }
    }

    /**
     * 通过门岗ID 和 网格类型 匹配 网格主键
     * @param map
     * @return
     */
    @Override
    public BigDecimal getGridIdByGatepostIdAndGridType(Map<String, Object> map) {
        return queuingRulesMapper.getGridIdByGatepostIdAndGridType(map);
    }

    /*=========================仓库排队规则=========================*/
    /**
     * 查询订单中所需要运输的物资  物资表对订单表 一对多
     * @param map 运输订单号 orderNumber
     * @return
     */
    @Override
    public List<Integer> getAllMaterialId(Map<String, Object> map) {
        return queuingRulesMapper.getAllMaterialId(map);
    }

    /**
     *通过物资Id查询仓库是否有货 或者 是否有多个仓库有货
     * @param materialId 物资ID
     * @return
     */
//    @Override
//    public List<Integer> getWarehouseIdByMaterialId(Integer materialId) {
//        return queuingRulesMapper.getWarehouseIdByMaterialId(materialId);
//    }

    /**
     * 通过仓库Id 和 物资Id 查询垛位 层次 月台 仓库
     * @param map MaterialId 物资ID warehouseId 仓库ID
     * @return
     */
//    @Override
//    public List<Map<String, Object>> getStackGradation(Map<String, Object> map) {
//        return queuingRulesMapper.getStackGradation(map);
//    }

    /**
     * 通过月台Id 查询月台当前排队车辆数
     * @param platformId 月台ID
     * @return
     */
//    @Override
//    public BigDecimal queueCountByPlatformId(BigDecimal platformId) {
//        return queuingRulesMapper.queueCountByPlatformId(platformId);
//    }


    /**
     * 获取月台Id 通过物资ID 根据排队规则 获取车辆该去哪个月台
     * 规则：首先遍历每一种物资 --> 不同物资在不同仓库 -->  遍历仓库通过物资Id和仓库Id查找对应的仓储网格表
     *       --> 找出多条数据 也就是说多个月台可以装  --> 遍历月台查看每个月台的排队车辆数 寻找最小的排队车辆数的月台
     *       --> 返回物资ID 和 月台ID
     * @param materialIdList
     * @return
     */
//    public List<Map<String, Object>> getPlatId(List<Integer> materialIdList){
//        //遍历每一中物资查找月台
//        List<Map<String, Object>> platIdList = new ArrayList<>();
//        con:for (Integer materialId : materialIdList) {
//            Integer platId = null;
//            //一种物资可能在多个仓库中
//            Map<String, Object> map = new HashMap<>();
//            map.put("materialId", materialId);
//
//            List<Integer> warehouseList = getWarehouseIdByMaterialId(materialId);
//            int countNum = 1000; //初始化比较值
//            for (Integer warehouseId : warehouseList) {
//                HashMap<String, Object> hashMap = new HashMap<>();
//                hashMap.put("materialId", materialId);
//                hashMap.put("warehouseId", warehouseId);
//                //通过物资Id和仓库ID对仓储网格表 再次进行查询并筛选出层次号最高的几条数据
//                List<Map<String, Object>> mesList = getStackGradation(hashMap);
//                //遍历筛选出来的月台 查看当前月台排队车辆数
//                for (Map<String, Object> mesMap : mesList) {
//                    BigDecimal platformId = DataChange.dataToBigDecimal(mesMap.get("platformId"));
//                    //通过月台ID 和 网格类型 查找当前网格Id
//                    hashMap.put("queueLocationId", platformId);
//                    hashMap.put("gridType", 1);
//                    BigDecimal gridId = queuingRulesMapper.getGridIdByGatepostIdAndGridType(hashMap);
//                    //取得排队车辆数
//                    int count = queueCountByPlatformId(gridId).intValue();
//                    //如果当前月台排队车数为0 则直接选择这个月台
//                    if(count == 0){
//                        platId = platformId.intValue();
//                        map.put("loadingId", platId);
//                        platIdList.add(map);
//                        continue con;
//                    }
//                    //如果排队车辆数小于上个月台排队车辆数
//                    if(count < countNum){
//                        countNum = count; //取代比较数
//                        platId = platformId.intValue(); //替换月台ID
//                        map.put("loadingId", platId);
//                    }
//                }
//            }
//            platIdList.add(map);
//        }
//        return platIdList;
//    }

}
